@@ -1,5 +1,5 @@
 package Terse;
-our $VERSION = '0.110';
+our $VERSION = '0.111';
 use 5.006;
 use strict;
 use warnings;
@@ -8,6 +8,7 @@ use Plack::Response;
 use JSON;
 use Scalar::Util qw/reftype/;
 use Time::HiRes qw(gettimeofday);
+use Terse::WebSocket;
 use Want qw/want/;
 use Digest::SHA;
 
@@ -16,7 +17,7 @@ BEGIN {
 	$JSON = JSON->new->utf8->canonical(1)->allow_blessed->convert_blessed;
 	%PRIVATE = (
 		map { $_ => 1 } 
-		qw/new run logger logInfo logError delayed_response content_type raiseError graft pretty serialize DESTROY TO_JSON AUTOLOAD/
+		qw/new run logger logInfo logError websocket delayed_response content_type raiseError graft pretty serialize DESTROY TO_JSON AUTOLOAD/
 	);
 }
 
@@ -44,9 +45,53 @@ sub run {
 		auth => 'auth',
 		insecure_session => 0,
 		content_type => 'application/json',
+		websocket => 1,
 		%args
 	);
 
+	$j->{websocket} = sub {
+		my ($self, %args) = @_;
+		my $env = $self->request->env;
+		if(!$env->{"psgi.streaming"}){ 
+			return $args{error}->($self, 'no psgi streaming');
+    		}
+		my $websocket = Terse::WebSocket->new($self);
+		$self->{_delayed_response} = sub {
+			my $responder = shift;
+			my @headers = $websocket->headers;
+			my $writer = eval { $responder->([101, [@headers]]); };
+			$args{($@ ? 'error' : 'connect')}->($websocket, $responder, $@);
+			$websocket->send('hello');
+			my $ping = 'ping';
+			my $reset_rate = $self->websocket_reset_rate ||= 100000;
+			eval {
+				my $ping_rate = $reset_rate;
+				while (1) {
+					$ping_rate--;
+					my $response;
+					if ($ping_rate < 0) {
+						$ping_rate = $reset_rate;
+						$websocket->send($ping);
+						$response = $websocket->recieve() while($ping_rate-- > 0 && !$response);
+						if (!$response || $response ne 'pong') {
+							last;
+						}
+						$ping_rate = $reset_rate;
+					}
+					$response = $websocket->recieve();
+					if ($response) {
+						$ping_rate = $reset_rate;
+						$args{recieve}->($websocket, $response, $responder);
+					}
+				}
+			};
+			$args{error}->($websocket, $responder, $@) if ($@);
+			$args{disconnect}->($websocket, $responder) if $args{disconnect};
+			$responder->([200, []]);
+		};
+		return $websocket;
+	} if $j->{_websocket};
+
 	$j->request = Plack::Request->new($args{plack_env});
 	$j->response = $pkg->new(
 		authenticated => \0,
@@ -181,7 +226,7 @@ sub _response {
 	$ct ||= 'application/json';
 	my $res = $self->{_delayed_response};
 	return $res if ($res); 
-	$res = $self->_build_response($sid, $ct, $status);
+	$res = $self->_build_response($sid, $ct, $status || 200);
 	$res->body($response_body->serialize());
 	return $res->finalize;
 }
@@ -192,7 +237,7 @@ sub _dispatch {
 		$self->{_application}->dispatch($method, $self, @params)
 	} : eval {
 		unless ($self->{_application}->can($method)) {
-			$self->response->raiseError('Invalid request', 400);
+			$self->response->raiseError('Invalid request - ' . $method, 400);
 			return;
 		}
 		$self->{_application}->$method($self, @params);
@@ -324,7 +369,7 @@ Terse - Lightweight Web Framework
 
 =head1 VERSION
 
-Version 0.110
+Version 0.111
 
 =cut
 
@@ -366,6 +411,22 @@ Version 0.110
 		});
 	}
 
+	sub websock {
+		my ($self, $t) = @_;
+		$t->websocket(
+			connect => {
+				my ($websocket) = @_;
+				$websocket->send('Hello');
+			},
+			recieve => {
+				my ($websocket, $message) = @_;
+				$websocket->send($message); # echo
+			},
+			error => { ... },
+			disconnect => { ... }
+		);
+	}
+
 	.... MyAPI.psgi ...
 
 	use Terse;
@@ -390,7 +451,7 @@ Version 0.110
 
 	....
 
-	plackup MyAPI.psgi
+	plackup -s Starman MyAPI.psgi
 
 	GET http://localhost:5000/?req=delayed_hello_world
 	# {"authenticated":1,"error":false,"errors":[],"hello":"world","status_code":200}
